###################################################################################################
# zmscustom.py
#
# Implementation of class ZMSCustom (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTMLFile
import string 
import urllib
import time
import sys
# Product Imports.
from zmscontainerobject import ZMSContainerObject
from zmsobject import ZMSObject
import zmsteaserelement
import _blobfields
import _globals


###################################################################################################
###################################################################################################
###   
###   C o n s t r u c t o r ( s )
###   
###################################################################################################
###################################################################################################

manage_addZMSCustomForm = HTMLFile('manage_addzmscustomform', globals()) 
def manage_addZMSCustom(self, meta_id, lang, manage_lang, _sort_id, REQUEST, RESPONSE):
  """ manage_addZMSCustom """
  
  if REQUEST['btn'] == self.getLangStr('BTN_INSERT',manage_lang):
    
    ##### Create ####
    id_prefix = _globals.id_prefix(REQUEST.get('id','e'))
    obj = ZMSCustom(self.getNewId(id_prefix),_sort_id+1,meta_id)
    self._setObject(obj.id, obj)
  
    obj = getattr(self,obj.id)
    ##### Object State ####
    obj.setObjStateNew(REQUEST)
    ##### Init Coverage ####
    coverage = self.getDCCoverage(REQUEST)
    if coverage.find('local.')==0:
      obj.setObjProperty('attr_dc_coverage',coverage)
    else:
      obj.setObjProperty('attr_dc_coverage','global.'+lang)
    ##### Init Properties ####
    obj.manage_changeProperties(lang,manage_lang,REQUEST,RESPONSE)
    ##### VersionManager ####
    obj.onChangeObj(REQUEST)
  
    ##### Normalize Sort-IDs ####
    self.normalizeSortIds(id_prefix)
  
    # Return with message.        
    message = self.getLangStr('MSG_INSERTED',manage_lang)%obj.display_type(REQUEST)
    RESPONSE.redirect('%s/%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(self.absolute_url(),obj.id,lang,manage_lang,urllib.quote(message)))

  else:
    RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s'%(self.absolute_url(),lang,manage_lang))


###################################################################################################
###################################################################################################
###   
###   C l a s s
###   
###################################################################################################
###################################################################################################

class ZMSCustom(ZMSContainerObject): 
        
    # Properties.
    # -----------
    meta_type = "ZMSCustom"
    icon = "misc_/zms/zmsdocument.gif"
    icon_disabled = "misc_/zms/zmsdocument_disabled.gif"

    # Management Options.
    # -------------------
    manage_options = ( 
	{'label': 'TAB_EDIT',       'action': 'manage_main'},
	{'label': 'TAB_ACCESS',     'action': 'manage_users'},
	{'label': 'TAB_REFERENCES', 'action': 'manage_RefForm'},
	{'label': 'TAB_HISTORY',    'action': 'manage_UndoVersionForm'},
	{'label': 'TAB_PREVIEW',    'action': 'preview_html'}, # empty string defaults to index_html
	)

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = (
		'manage','manage_main',
		'manage_properties','manage_changeProperties',
		'manage_moveObjUp','manage_moveObjDown',
		'manage_cutObjects','manage_copyObjects','manage_pasteObjs',
		'manage_rollbackVersionChanges','manage_requestVersionChanges',
		)
    __editorPermissions__ = (
		'manage_commitVersionChanges',
		)
    __administratorPermissions__ = (
		)
    __ac_permissions__=(
		('ZMS Author', __authorPermissions__),
		('ZMS Editor', __editorPermissions__),
		('ZMS Administrator', __administratorPermissions__),
		)

    # Properties.
    # -----------
    __obj_attrs__ = {
        # Changed by
        'change_uid':{'datatype':'string','multilang':1,'xml':0,'lang_inherit':_globals.LI_FORBIDDEN},
        'change_dt':{'datatype':'datetime','multilang':1,'xml':0,'lang_inherit':_globals.LI_FORBIDDEN},
        # Versioned by
        'work_uid':{'datatype':'string','multilang':1,'xml':0,'lang_inherit':_globals.LI_FORBIDDEN},
        'work_dt':{'datatype':'datetime','multilang':1,'xml':0,'lang_inherit':_globals.LI_FORBIDDEN},
        # Properties
        'sort_id':{'datatype':'string','xml':0},
        'active':{'datatype':'boolean','multilang':1},
        'attr_active_start':{'datatype':'datetime','multilang':1},
        'attr_active_end':{'datatype':'datetime','multilang':1},
        # Meta-Data
        'attr_dc_coverage':{'datatype':'string'},
    }


    # Management Interface.
    manage_main = HTMLFile('dtml/metaobj/manage_main', globals())


    """
    ###############################################################################################    
    #
    #   CONSTRUCTOR
    #
    ###############################################################################################    
    """

    ###############################################################################################
    # ZMSCustom.__init__: 
    #
    # Constructor (initialise a new instance of ZMSCustom).
    ###############################################################################################
    def __init__(self, id='', sort_id='', meta_id=''): 
      """ ZMSCustom.__init__ """
      ZMSObject.__init__(self,id,sort_id)
      self.meta_id = meta_id


    # ------------------------------------------------------------------------------------------
    #  ZMSCustom.recurse_updateVersion:
    #
    #  Update version.
    # ------------------------------------------------------------------------------------------
    def recurse_updateVersion(self, REQUEST):
      message = ''

      # Process super-objects.
      message = message + ZMSObject.recurse_updateVersion(self,REQUEST)
        
      # Process object.
      # [...]

      # Recurse through child-nodes.
      for ob in self.objectValues(self.dGlobalAttrs.keys()):
        message = message + ob.recurse_updateVersion(REQUEST)
      
      # Return with message.
      return message


    """
    ###############################################################################################    
    #
    #   Eigenschaften
    #
    ###############################################################################################    
    """

    ###############################################################################################
    #  ZMSCustom.manage_changeProperties: 
    #
    #  Change Custom properties.
    ###############################################################################################
    def manage_changeProperties(self, lang, manage_lang, REQUEST, RESPONSE):
        """ ZMSCustom.manage_changeProperties """
        message = ''
        
        redirect_self = 0
        for attr in self.getMetaobj(self.meta_id)['attrs']:
           attr_type = attr['type']
           redirect_self = redirect_self or attr_type in self.dGlobalAttrs.keys()
           redirect_self = redirect_self or attr_type in self.getMetaobjIds()
        redirect_self = redirect_self and not REQUEST.get('btn','') == self.getLangStr('BTN_CANCEL',manage_lang)
           
        if REQUEST.get('btn','') != self.getLangStr('BTN_CANCEL',manage_lang):
          
          ##### Object State #####
          self.setObjStateModified(REQUEST)
          
          ##### Properties #####
          for key in self.getObjAttrs().keys():
            self.setReqProperty(key,REQUEST)
          
          ##### Blobs ####
          _blobfields.thumbnailImageFields(self,lang,manage_lang,REQUEST)
          _blobfields.synchronizeBlobFields(self)
          
          ##### VersionManager ####
          self.onChangeObj(REQUEST)
          
          ##### Message #####
          message = self.getLangStr('MSG_CHANGED',manage_lang)

        # Return with message.
        if redirect_self:          
          return RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,urllib.quote(message)))
        else:        
          self.checkIn(REQUEST)
          return RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s#_%s'%(self.getParentNode().absolute_url(),lang,manage_lang,urllib.quote(message),self.id))


    # -------------------------------------------------------------------------------------------------
    #	ZMSCustom.isMetaType:
    # -------------------------------------------------------------------------------------------------
    def isMetaType(self, meta_type, REQUEST={}):
      value = 0
      # ZMSCustom.
      if type(meta_type) is type(''):
        value = value or (self.meta_type == meta_type or self.meta_id == meta_type)
      elif type(meta_type) == type([]):
        value = value or (self.meta_type in meta_type or self.meta_id in meta_type)
      # ZMSContainerObject.
      if not value:
        value = value or ZMSContainerObject.isMetaType(self,meta_type,REQUEST)
      return value
    
    # -------------------------------------------------------------------------------------------------
    #  ZMSCustom.isPage:
    # -------------------------------------------------------------------------------------------------
    def isPage(self): return self.getType() == 'ZMSDocument'

    # -------------------------------------------------------------------------------------------------
    #  ZMSObject.isPageElement:
    # -------------------------------------------------------------------------------------------------
    def isPageElement(self): return self.getType() == 'ZMSObject'

    # ---------------------------------------------------------------------------------------------
    #  ZMSCustom.getType
    # ---------------------------------------------------------------------------------------------
    def getType(self):
      docElmnt = self.getDocumentElement()
      metaObj = docElmnt.getMetaobj(self.meta_id)
      return metaObj['type']

    # ---------------------------------------------------------------------------------------------
    #  ZMSCustom.getPenetrance
    # ---------------------------------------------------------------------------------------------
    def getPenetrance(self, REQUEST):
      penetrance = 0
      if 'attr_penetrance' in self.getObjAttrs().keys():
        i = self.getObjProperty('attr_penetrance',REQUEST)
        lst = zmsteaserelement.lstPenetrance
        if i in lst:
          penetrance = lst[lst.index(i)-1]
      return penetrance

    # ---------------------------------------------------------------------------------------------
    #  ZMSCustom.getDtmlTemplate
    # ---------------------------------------------------------------------------------------------
    def getDtmlTemplate(self):
      docElmnt = self.getDocumentElement()
      metaObj = docElmnt.getMetaobj(self.meta_id)
      tmplt = metaObj['tmplt']
      dtml_tmplt = getattr(docElmnt,tmplt,None)
      return dtml_tmplt

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.getTitlealt
    # ---------------------------------------------------------------------------------------------
    def getTitlealt(self,REQUEST):
      s = ""
      if self.isPage():
        metaObjAttrs = self.getMetaobj(self.meta_id)['attrs']
        i = 0
        while 1:
          if i >= len(metaObjAttrs) or len(s) > 0:
            break;
          objAttr = self.getObjAttr(metaObjAttrs[i]['id'])
          if objAttr['datatype_key']==_globals.DT_STRING:
            s = str(self.getObjProperty(objAttr['key'],REQUEST))
          i = i + 1
      if len(s) == 0: 
        s = self.display_type(REQUEST)
      s = self.getLevelnfc(REQUEST) + s
      return s

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.getTitle
    # ---------------------------------------------------------------------------------------------
    def getTitle(self,REQUEST): 
      s = ""
      if self.isPage():
        metaObjAttrs = self.getMetaobj(self.meta_id)['attrs']
        i = 1
        while 1:
          if i >= len(metaObjAttrs) or len(s) > 0:
            break;
          objAttr = self.getObjAttr(metaObjAttrs[i]['id'])
          if objAttr['datatype_key']==_globals.DT_STRING:
            s = str(self.getObjProperty(objAttr['key'],REQUEST))
          i = i + 1
      if len(s) == 0: 
        s = self.display_type(REQUEST)
      s = self.getLevelnfc(REQUEST) + s
      return s


    """
    ###############################################################################################    
    ###  
    ###  HTML-Darstellung 
    ### 
    ###############################################################################################    
    """

    # preload display interface for custom-objects
    render_default = HTMLFile('dtml/metaobj/render_default', globals()) 

    # ---------------------------------------------------------------------------------------------
    #  ZMSCustom._getBodyContent:
    #
    #  HTML presentation of Special Document. 
    # ---------------------------------------------------------------------------------------------
    def _getBodyContent(self, REQUEST, thumbnail=0):
      dtml_tmplt = self.getDtmlTemplate()
      if self.getType() == 'ZMSObject':
        return dtml_tmplt(self,REQUEST)
      elif self.getType() in ['ZMSDocument','ZMSResource']:
        return dtml_tmplt(self,REQUEST)
      elif self.getType() == 'ZMSTeaserElement':
        s = ""
        s = s + '<table cellspacing="0" cellpadding="0" border="0" width="100%" class="teaserElement">\n'
        s = s + '<tr'
        if REQUEST.has_key('bgcolor_text'):
          s = s + ' class="%s"'%str(REQUEST['bgcolor_text'])
        s = s + '>\n'
        s = s + '<td align="left" valign="top" class="teaserText">'
        s = s + '<p class="teaserText">%s</p>'%str(dtml_tmplt(self,REQUEST))
        s = s + '</td>\n'
        s = s + '</tr>\n'
        s = s + '</table>\n'
        return s

    # ---------------------------------------------------------------------------------------------
    #  ZMSCustom.renderShort:
    #
    #  Renders short presentation of Special Document.
    # ---------------------------------------------------------------------------------------------
    def renderShort(self, REQUEST):
      html = ""
      try:
        if self.getType() in ['ZMSDocument','ZMSResource']:
          html = html + self.render_default(self,ob=self,REQUEST=REQUEST)
        else:
          html = html + self._getBodyContent(REQUEST)
        # Process html <form>-tags.
        html = _globals.form_quote(html,REQUEST)
      except:
        html = html + '[Exception]:<br>'
        html = html + '-  Type: <strong>%s</strong><br>'%str(sys.exc_type)
        html = html + '-  Value: <strong>%s</strong><br>'%str(sys.exc_value)
        html = html + '-  Traceback: <strong>%s</strong>'%str(sys.exc_traceback)
      # Return <html>.
      return html

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.printHtml:
    #
    #	Renders print presentation of Special Document.
    # ---------------------------------------------------------------------------------------------
    def printHtml(self, level, sectionize, sectionizer, REQUEST):
      html = ""
    
      # Title.
      title = self.getTitle(REQUEST)
      sectionizer.processLevel(level)
      if sectionize:
        title = '%s %s'%(str(sectionizer),title)
      html = html + "<h%i>%s</h%i>"%(level,title,level)

      # bodyContent_PagePrePDF
      if self.getType() == 'ZMSDocument':
        if hasattr(self,'bodyContent_PagePrePDF'):
          html = html + self.bodyContent_PagePrePDF(self,REQUEST)
    
      # bodyContent
      html = html + self._getBodyContent(REQUEST)
        
      # bodyContent_PagePostPDF
      if self.getType() == 'ZMSDocument':
        if hasattr(self,'bodyContent_PagePostPDF'):
          html = html + self.bodyContent_PagePostPDF(self,REQUEST)
        
      # Return <html>.
      return html

###################################################################################################
